/*
/ Classdef.h
/ class definitions for spatialite-gui  - a SQLite /SpatiaLite GUI tool
/
/ version 1.1, 2008 September 13
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2008  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU General Public License for more details.
/
/    You should have received a copy of the GNU General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include "wx/wx.h"
#include "wx/aui/aui.h"
#include "wx/treectrl.h"
#include "wx/grid.h"
#include "wx/listctrl.h"
#include "wx/propdlg.h"
#include "wx/generic/propdlg.h"

#include <spatialite/sqlite3.h>

#define OMIT_GEOS    0
#define OMIT_PROJ    0
#include <spatialite/gaiageo.h>

enum
{
// control IDs for main window and tree list control
    ID_Connect = 1,
    ID_CreateNew,
    ID_Disconnect,
    ID_Vacuum,
    ID_SqlScript,
    ID_LoadShp,
	ID_LoadTxt,
    ID_VirtualShp,
	ID_VirtualTxt,
    ID_Srids,
    ID_Charset,
    ID_Help,
    Tree_NewTable,
    Tree_NewView,
    Tree_NewIndex,
    Tree_NewTrigger,
    Tree_NewColumn,
    Tree_Show,
    Tree_Drop,
    Tree_Rename,
    Tree_Select,
    Tree_Refresh,
    Tree_SpatialIndex,
	Tree_MbrCache,
    Tree_ShowSql,
    Tree_Recover,
    Tree_CheckGeometry,
    Tree_SetSrid,
    Tree_DumpShp,
    Tree_DumpTxtTab,
    Tree_DumpCsv,
    Tree_DumpHtml,
    Tree_Edit,
    Grid_Clear,
    Grid_All,
    Grid_Column,
    Grid_Row,
    Grid_Copy,
    Grid_Blob,
    Grid_Delete,
    Grid_Insert,
    Grid_Abort,
    Grid_BlobIn,
    Grid_BlobOut,
    Grid_BlobNull,
    Image_Copy,
};

enum
{
// control IDs for dialogs
    ID_SQL = 10000,
    ID_SQL_GO,
    ID_HISTORY_BACK,
    ID_HISTORY_FORWARD,
    ID_RS_FIRST,
    ID_RS_LAST,
    ID_RS_NEXT,
    ID_RS_PREVIOUS,
    ID_REFRESH,
    ID_RS_BLOCK,
    ID_PANE_HEXADECIMAL,
    ID_PANE_GEOMETRY,
    ID_PANE_IMAGE,
    ID_HEX,
    ID_GEOM_TABLE,
    ID_GEOM_GRAPH,
    ID_GEOM_BOX,
    ID_IMAGE_TITLE,
    ID_IMG_BOX,
    ID_IMAGE,
	ID_VIRTSHP_TABLE,
    ID_VIRTSHP_SRID,
    ID_VIRTSHP_CHARSET,
	ID_VIRTTXT_TABLE,
	ID_VIRTTXT_CHARSET,
	ID_VIRTTXT_TITLES,
	ID_VIRTTXT_SEPARATOR,
	ID_VIRTTXT_CHARSEPARATOR,
	ID_VIRTTXT_QUOTE,
	ID_VIRTTXT_POINT,
    ID_LDSHP_TABLE,
    ID_LDSHP_COLUMN,
    ID_LDSHP_SRID,
    ID_LDSHP_CHARSET,
	ID_LDTXT_TABLE,
	ID_LDTXT_CHARSET,
	ID_LDTXT_TITLES,
	ID_LDTXT_SEPARATOR,
	ID_LDTXT_CHARSEPARATOR,
	ID_LDTXT_QUOTE,
	ID_LDTXT_POINT,
    ID_DMPSHP_CHARSET,
    ID_DMPTXT_CHARSET,
    ID_DFLT_CHARSET,
    ID_DFLT_ASK,
    ID_SCRIPT_CHARSET,
    ID_RCVR_SRID,
    ID_RCVR_TYPE,
    ID_SRID_OLD,
    ID_SRID_SRID,
    ID_SEARCH,
};

enum
{
// tree item data types
    MY_TABLE = 0,
    MY_VTABLE,
    MY_VIEW,
    MY_COLUMN,
    MY_GEOMETRY,
    MY_GEOMETRY_INDEX,
	MY_GEOMETRY_CACHED,
    MY_INDEX,
    MY_TRIGGER,
    MY_INT_VARIANT,
    MY_DBL_VARIANT,
    MY_TXT_VARIANT,
    MY_BLOB_VARIANT,
    MY_NULL_VARIANT,
};

class MyObject:public wxTreeItemData
{
//
// a class to store TreeItemData
//
  private:
    int Type;			// the object type
    wxString Name;		// the object name
    wxString Column;		// the column name [optional]
  public:
      MyObject (int type, wxString & name);
      MyObject (int type, wxString & name, wxString & column);
      virtual ~ MyObject () { ; }
    int GetType () { return Type; }
    wxString & GetName () {	return Name; }
    wxString & GetColumn () { return Column; }
};

class MyColumnInfo
{
//
// a class to store a DB column
//
  private:
    wxString Name;		// the column name
    bool PrimaryKey;		// Primary Key column
    bool Geometry;		// Geometry column
    bool GeometryIndex;		// Geometry column + SpatialIndex
	bool MbrCache;		// Geometry column + MbrCache
    MyColumnInfo *Next;		// pointer to next element into the linked list
  public:
      MyColumnInfo (wxString & name, bool pkey);
     ~MyColumnInfo () { ; }
    wxString & GetName () { return Name; }
    bool IsPrimaryKey () { return PrimaryKey; }
    void SetGeometry () { Geometry = true; }
    bool IsGeometry () { return Geometry; }
    void SetGeometryIndex () { GeometryIndex = true; }
    bool IsGeometryIndex () { return GeometryIndex; }
	void SetMbrCache () { MbrCache = true; }
    bool IsMbrCache () { return MbrCache; }
    void SetNext (MyColumnInfo * next) { Next = next; }
    MyColumnInfo *GetNext () { return Next; }
};

class MyIndexInfo
{
//
// a class to store a DB index
//
  private:
    wxString Name;		// the index name
    MyIndexInfo *Next;		// pointer to next element into the linked list
  public:
      MyIndexInfo (wxString & name);
     ~MyIndexInfo () { ; }
    wxString & GetName () { return Name; }
    void SetNext (MyIndexInfo * next) { Next = next; }
    MyIndexInfo *GetNext () { return Next; }
};

class MyTriggerInfo
{
//
// a class to store a DB trigger
//
  private:
    wxString Name;		// the trigger name
    MyTriggerInfo *Next;	// pointer to next element into the linked list
  public:
      MyTriggerInfo (wxString & name);
     ~MyTriggerInfo () { ; }
    wxString & GetName () { return Name; }
    void SetNext (MyTriggerInfo * next) { Next = next; }
    MyTriggerInfo *GetNext () { return Next; }
};

class MyTableInfo
{
//
// a class to store DB table columns
//
  private:
    MyColumnInfo * FirstColumn;	// first element into the columns linked list
    MyColumnInfo *LastColumn;	// last element into the columns linked list
    MyIndexInfo *FirstIndex;	// first element into the indices linked list
    MyIndexInfo *LastIndex;	// last element into the indices linked list
    MyTriggerInfo *FirstTrigger;	// first element into the triggers linked list
    MyTriggerInfo *LastTrigger;	// last element into the triggers linked list
  public:
      MyTableInfo () { FirstColumn = NULL; LastColumn = NULL;
		FirstIndex = NULL; LastIndex = NULL; FirstTrigger = NULL;
		LastTrigger = NULL; }
     ~MyTableInfo ();
    void AddColumn (wxString & name, bool pkey);
    void SetGeometry (wxString & name, bool index, bool cached);
    void AddIndex (wxString & name);
    void AddTrigger (wxString & name);
    MyColumnInfo *GetFirstColumn () { return FirstColumn; }
    MyIndexInfo *GetFirstIndex () { return FirstIndex; }
    MyTriggerInfo *GetFirstTrigger () { return FirstTrigger; }
};

class MyViewInfo
{
//
// a class to store DB view columns
//
  private:
    MyColumnInfo * First;	// first element into the columns linked list
    MyColumnInfo *Last;		// last element into the columns linked list
  public:
      MyViewInfo () { First = NULL; Last = NULL; }
     ~MyViewInfo ();
    void AddColumn (wxString & name);
    MyColumnInfo *GetFirst () { return First; }
};

class MyVariant
{
//
// a class to store Variant-Type values
//
  private:
    int Type;			// the Variant-Type
    int IntValue;		// the Integer value
    double DblValue;		// the Double value
    wxString TxtValue;		// the Text value
    unsigned char *Blob;	// the BLOB value
    int BlobSize;		// the BLOB size
  public:
      MyVariant () { Type = MY_NULL_VARIANT; Blob = NULL; }
     ~MyVariant () { if (Blob) delete Blob; }
    void Clear () { if (Blob) delete Blob; Blob = NULL;
		Type = MY_NULL_VARIANT; }
    void Set (int value) { Type = MY_INT_VARIANT;
		IntValue = value; }
    void Set (double value) { Type = MY_DBL_VARIANT;
		DblValue = value; }
    void Set (const unsigned char *text);
    void Set (wxString & string) { Type = MY_TXT_VARIANT;
		TxtValue = string; }
    void Set (const void *blob, int size);
    void Copy (MyVariant * other);
    int GetType () { return Type; }
    int GetIntValue () { return IntValue; }
    double GetDblValue () { return DblValue; }
    wxString & GetTxtValue () { return TxtValue; }
    int GetBlobSize () { return BlobSize; }
    unsigned char *GetBlob () { return Blob; }
};

class MyRowVariant
{
//
// a class to store a row composed of Variant-Type values
//
  private:
    int NumCols;		// number of columns
    MyVariant *ColumnArray;	// the column as an array
    bool Deleted;		// switch to mark row deletion
    MyRowVariant *Next;		// pointer to next element into the linked list
  public:
      MyRowVariant () { NumCols = 0; ColumnArray = NULL;
		Deleted = false; Next = NULL; }
    MyRowVariant (int cols) { NumCols = cols;
		ColumnArray = new MyVariant[cols]; Next = NULL; }
    ~MyRowVariant () { if (ColumnArray) delete[]ColumnArray; }
    void Create (int cols);
    int GetNumCols () { return NumCols; }
    void Set (int col, int value);
    void Set (int col, double value);
    void Set (int col, const unsigned char *text);
    void Set (int col, const void *blob, int size);
    MyVariant *GetColumn (int col);
    void SetDeleted () { Deleted = true; }
    bool IsDeleted () { return Deleted; }
    void SetNext (MyRowVariant * next) { Next = next; }
    MyRowVariant *GetNext () { return Next; }
};

class MyVariantList
{
//
// a class to store a whole result set
//
  private:
    int NumCols;		// number of columns
    wxString *ColumnName;	// the column names
    MyRowVariant *First;	// first element into the linked list
    MyRowVariant *Last;		// last element into the linked list
  public:
      MyVariantList ();
     ~MyVariantList ();
    MyRowVariant *Add (int columns);
    void SetColumnName (int col, const char *colName);
    MyRowVariant *GetFirst () { return First; }
    int GetRows (); 
    int GetColumns () { return NumCols; }
    wxString & GetColumnName (int col);
};

class MyBlobs
{
//
// a class to store BLOBs
//
  private:
    int NumRows;		// the number of rows
    int NumCols;		// the number of columns
    MyRowVariant *Rows;		// pointer to an array of rows
  public:
      MyBlobs (int rows, int cols);
     ~MyBlobs ();
    void SetBlob (int row, int col, MyVariant * blobVar);
    MyVariant *GetBlob (int row, int col);
};

class MyValues
{
//
// a class to store column values for editing
//
  private:
    int NumRows;		// the number of rows
    int NumCols;		// the number of columns
    MyRowVariant *Rows;		// pointer to an array of rows
  public:
      MyValues (int rows, int cols);
     ~MyValues ();
    void SetValue (int row, int col, int value);
    void SetValue (int row, int col, double value);
    void SetValue (int row, int col, wxString & string);
    MyRowVariant *GetRow (int row);
    MyVariant *GetValue (int row, int col);
};

class MySqlQuery
{
//
// a class to store an SQL query - history
//
  private:
    wxString Sql;
    MySqlQuery *Prev;
    MySqlQuery *Next;
  public:
      MySqlQuery (wxString & sql) { Sql = sql; Prev = NULL; 
		Next = NULL; }
     ~MySqlQuery () { ; }
    wxString & GetSql () { return Sql; }
    void SetPrev (MySqlQuery * prev) { Prev = prev; }
    MySqlQuery *GetPrev () { return Prev; }
    void SetNext (MySqlQuery * next) { Next = next; }
    MySqlQuery *GetNext () { return Next; }
};

class MySqlHistory
{
//
// a class supporting SQL queries history
//
  private:
    MySqlQuery * First;
    MySqlQuery *Last;
    MySqlQuery *Current;
  public:
      MySqlHistory () { First = NULL; Last = NULL;
		Current = NULL; }
     ~MySqlHistory ();
    void Add (wxString & sql);
    MySqlQuery *GetCurrent () { return Current; }
    MySqlQuery *GetNext ();
    MySqlQuery *GetPrev ();
    bool TestNext ();
    bool TestPrev ();
};

class MyApp:public wxApp
{
//
// the main APP
//
    virtual bool OnInit ();
};

class MyTableTree:public wxTreeCtrl
{
//
// a tree-control used for SQLite DB tables
//
  private:
    class MyFrame * MainFrame;
    wxTreeItemId Root;		// the root node
    wxImageList *Images;	// the images list
    wxTreeItemId CurrentItem;	// the tree item holding the current context menu
  public:
      MyTableTree () { ; }
    MyTableTree (MyFrame * parent, wxWindowID id = wxID_ANY);
    virtual ~ MyTableTree () { ; }
    void SetPath (wxString & path) { SetItemText (Root, path); }
    void FlushAll () { DeleteChildren (Root); }
    void AddTable (wxString & tableName, bool virtualTable);
    void AddView (wxString & viewName);
    void ExpandRoot () { Expand (Root); }
    void ShowMenu (wxTreeItemId & item, wxPoint & pt);
    void OnSelChanged (wxTreeEvent & event);
    void OnRightClick (wxTreeEvent & event);
    void OnCmdNewTable (wxCommandEvent & event);
    void OnCmdNewView (wxCommandEvent & event);
    void OnCmdNewIndex (wxCommandEvent & event);
    void OnCmdNewTrigger (wxCommandEvent & event);
    void OnCmdNewColumn (wxCommandEvent & event);
    void OnCmdShow (wxCommandEvent & event);
    void OnCmdDrop (wxCommandEvent & event);
    void OnCmdRename (wxCommandEvent & event);
    void OnCmdSelect (wxCommandEvent & event);
    void OnCmdRefresh (wxCommandEvent & event);
    void OnCmdRecover (wxCommandEvent & event);
    void OnCmdShowSql (wxCommandEvent & event);
    void OnCmdSpatialIndex (wxCommandEvent & event);
	void OnCmdMbrCache (wxCommandEvent & event);
    void OnCmdCheckGeometry (wxCommandEvent & event);
    void OnCmdSetSrid (wxCommandEvent & event);
    void OnCmdDumpShp (wxCommandEvent & event);
    void OnCmdDumpTxtTab (wxCommandEvent & event);
    void OnCmdDumpCsv (wxCommandEvent & event);
    void OnCmdDumpHtml (wxCommandEvent & event);
    void OnCmdEdit (wxCommandEvent & event);
};

class MyQueryView:public wxPanel
{
//
// a panel to be used for SQL Queries
//
  private:
    class MyFrame * MainFrame;
    MySqlHistory History;
    wxTextCtrl *SqlCtrl;
    wxBitmapButton *BtnSqlGo;
    wxBitmapButton *BtnHistoryBack;
    wxBitmapButton *BtnHistoryForward;
    wxBitmapButton *BtnRsFirst;
    wxBitmapButton *BtnRsLast;
    wxBitmapButton *BtnRsNext;
    wxBitmapButton *BtnRsPrevious;
    wxBitmapButton *BtnRefresh;
    wxStaticText *RsCurrentBlock;
    int RsBlock;
    int RsBeginRow;
    int RsEndRow;
    int RsMaxRow;
    int CurrentEvtRow;
    int CurrentEvtColumn;
    MyVariant *CurrentBlob;
    wxString SqlErrorMsg;
    wxGrid *TableView;
    MyBlobs *TableBlobs;
    MyValues *TableValues;
    bool ReadOnly;
    int *RowIds;
    int PrimaryKeys[1024];
    int BlobColumns[1024];
    wxString TableName;
    bool InsertPending;
    MyRowVariant *InsertRow;
  public:
      MyQueryView () { ; }
    MyQueryView (MyFrame * parent, wxWindowID id = wxID_ANY);
    virtual ~ MyQueryView () { if (RowIds) delete[]RowIds;
		if (InsertRow) delete InsertRow; }
    void HideControls ();
    void HideRsControls ();
    void ShowSqlControls ();
    void ShowRsControls ();
    bool IsPrimaryKey (int column);
    bool IsBlobColumn (int column);
    void EditTable (wxString & sql, int *primaryKeys, int *blobCols,
		    wxString & tableName);
    void SetSql (wxString & sql, bool execute);
    bool ExecuteSql (wxString & sql, int from);
    void CreateGrid (int rows, int cols);
    void ResizeTableView ();
    void SetHistoryStates ();
    void DoInsert (bool confirmed);
    void HexBlobValue (unsigned char *blob, int size, wxString & hex);
    void OnSize (wxSizeEvent & event);
    void OnSqlGo (wxCommandEvent & event);
    void OnHistoryBack (wxCommandEvent & event);
    void OnHistoryForward (wxCommandEvent & event);
    void OnRsFirst (wxCommandEvent & event);
    void OnRsLast (wxCommandEvent & event);
    void OnRsNext (wxCommandEvent & event);
    void OnRsPrevious (wxCommandEvent & event);
    void OnRefresh (wxCommandEvent & event);
    void OnCellSelected (wxGridEvent & event);
    void OnRightClick (wxGridEvent & event);
    void OnCellChanged (wxGridEvent & event);
    void OnCmdDelete (wxCommandEvent & event);
    void OnCmdInsert (wxCommandEvent & event);
    void OnCmdAbort (wxCommandEvent & event);
    void OnCmdClearSelection (wxCommandEvent & event);
    void OnCmdSelectAll (wxCommandEvent & event);
    void OnCmdSelectRow (wxCommandEvent & event);
    void OnCmdSelectColumn (wxCommandEvent & event);
    void OnCmdCopy (wxCommandEvent & event);
    void OnCmdBlob (wxCommandEvent & event);
    void OnCmdBlobIn (wxCommandEvent & event);
    void OnCmdBlobOut (wxCommandEvent & event);
    void OnCmdBlobNull (wxCommandEvent & event);
};

class MyFrame:public wxFrame
{
//
// the main GUI frame
//
  private:
    unsigned char Jpeg1Signature[2];
    unsigned char Jpeg2Signature[2];
    unsigned char Jpeg3Signature[4];
    unsigned char JfifSignature[4];
    unsigned char ExifSignature[4];
    unsigned char PngSignature[8];
    unsigned char ZipSignature[4];
    wxAuiManager Manager;	// the GUI manager
    MyTableTree *TableTree;	// the tables tree list
    MyQueryView *QueryView;	// the QueryResult panel
    bool HelpPane;		// is the HELP pane already opened ?
    sqlite3 *SqliteHandle;	// handle for SQLite DB
    wxString SqlitePath;	// path of SQLite DB
    wxString LastDirectory;	// path of directory used  
    int CharsetsLen;		// # charsets defined
    wxString *Charsets;		// table of charsets [code only]
    wxString *CharsetsNames;	// table of charsets [whith description]
    wxString LocaleCharset;	// locale charset
    wxString DefaultCharset;	// default charset
    bool AskCharset;		// switch to set default charset for every output
    wxBitmap *BtnCreateNew;	// button icon for DB CREATE&CONNECT
    wxBitmap *BtnConnect;	// button icon for DB CONNECT
    wxBitmap *BtnDisconnect;	// button icon for DB DISCONNECT
    wxBitmap *BtnVacuum;	// button icon for DB VACUUM
    wxBitmap *BtnSqlScript;	// button icon for Execute SQL SCRIPT
    wxBitmap *BtnCharset;	// button icon for Default CHARSET
    wxBitmap *BtnLoadShp;	// button icon for LOAD SHP
	wxBitmap *BtnLoadTxt;	// button icon for LOAD TXT/CSV
    wxBitmap *BtnVirtualShp;	// button icon for VIRTUAL SHP
	wxBitmap *BtnVirtualTxt;	// button icon for VIRTUAL TXT/CSV
    wxBitmap *BtnSrids;		// button icon for SEARCH SRIDs
    wxBitmap *BtnHelp;		// button icon for HELP
    wxBitmap *BtnAbout;		// button icon for ABOUT
    wxBitmap *BtnExit;		// button icon for EXIT
  public:
    enum
    {
	BLOB_NULL = 0,
	BLOB_HEX,
	BLOB_GEOMETRY,
	BLOB_JPEG,
	BLOB_JFIF,
	BLOB_EXIF,
	BLOB_PNG,
	BLOB_GIF,
	BLOB_PDF,
	BLOB_ZIP,
    };
      MyFrame (const wxString & title, const wxPoint & pos, const wxSize & size,
	       wxString & path);
      virtual ~ MyFrame ();

    void UpdateStatusBar ();
    bool IsConnected () { if (SqliteHandle) return true;
		else return false;
    }
    static void CleanSql (char *str);
    void CleanTxtTab (char *str);
    void CleanCsv (char *str);
    void CleanHtml (char *str);
    sqlite3 *GetSqlite () { return SqliteHandle; }
    void CloseHelpPane () { HelpPane = false; }
    void OpenHelpPane () { HelpPane = true; }

    bool OpenDB ();
    bool CreateDB ();
    void CloseDB ();
    void InitTableTree ();
    void ClearTableTree ();
    void GetTableColumns (wxString & tableName, MyTableInfo * list);
    void GetTableIndices (wxString & tableName, MyTableInfo * list);
    void GetTableTriggers (wxString & tableName, MyTableInfo * list);
    void GetViewColumns (wxString & viewName, MyViewInfo * list);
    void EditTable (wxString & sql, int *primaryKeys, int *blobCols,
		    wxString & table) 
			{ QueryView->EditTable (sql, primaryKeys, blobCols, table); }
    void SetSql (wxString & sql, bool execute)
		{ QueryView->SetSql (sql, execute); }
    int GuessBlobType (int blob_size, unsigned char *blob);
    bool ExecuteSql (const char *sql, int rowNo);
    void Rollback ();
    bool TableAlreadyExists (wxString & name);
    bool SridNotExists (int srid);
    bool CheckMetadata ();
    void CleanSqlString (char *sql);
    wxString *GetCharsets () { return Charsets; }
    wxString *GetCharsetsNames () { return CharsetsNames; }
    int GetCharsetsLen () { return CharsetsLen; }
    gaiaDbfFieldPtr GetDbfField (gaiaDbfListPtr list, char *name);
	void LoadShapefile (wxString & path, wxString & table, int srid,
			wxString & column, wxString & charset);
    void DumpShapefile (wxString & path, wxString & table, wxString & column,
			wxString & charset);
	void LoadText (wxString & path, wxString & table, wxString & charset, bool first_titles, char decimal_separator, 
	char separator, char text_separator);
	void DumpTxtTab (wxString & path, wxString & table, wxString & charset);
    void DumpCsv (wxString & path, wxString & table, wxString & charset);
    void DumpHtml (wxString & path, wxString & table, wxString & charset);
    void GetHelp (wxString & html);
    wxString *GetColumnNames (wxString & table, int *columns);
    void SetLastDirectory (wxString & path) { LastDirectory = path; }
    wxString & GetLastDirectory () { return LastDirectory; }
    wxString & GetLocaleCharset () { return LocaleCharset; }
    wxString & GetDefaultCharset () { return DefaultCharset; }
    wxString & GetCharsetName (wxString & charset);
    int GetCharsetIndex (wxString & charset);
    bool IsSetAskCharset () { return AskCharset; }
	char * ReadSqlLine(FILE *fl, int *len, int *eof);
	
    void OnQuit (wxCommandEvent & event);
    void OnAbout (wxCommandEvent & event);
    void OnConnect (wxCommandEvent & event);
    void OnCreateNew (wxCommandEvent & event);
    void OnDisconnect (wxCommandEvent & event);
    void OnVacuum (wxCommandEvent & event);
    void OnSqlScript (wxCommandEvent & event);
    void OnCharset (wxCommandEvent & event);
    void OnLoadShp (wxCommandEvent & event);
	void OnLoadTxt (wxCommandEvent & event);
    void OnVirtualShp (wxCommandEvent & event);
	void OnVirtualTxt (wxCommandEvent & event);
    void OnSrids (wxCommandEvent & event);
    void OnHelp (wxCommandEvent & event);
    void OnMouseMove (wxMouseEvent & event);
};

class HelpDialog:public wxDialog
{
//
// the help dialog
//
  private:
    MyFrame * MainFrame;
  public:
    HelpDialog () { MainFrame = NULL; }
    HelpDialog (MyFrame * parent) { Create (parent); }
    bool Create (MyFrame * parent);
    virtual ~ HelpDialog () { ; }
    void CreateControls ();
    void OnClose (wxCloseEvent & event);
};

class SearchSridDialog:public wxDialog
{
//
// a dialog preparing a Search SRID by name
//
  private:
    MyFrame * MainFrame;
    wxString String;		// required search string
  public:
      SearchSridDialog () { ; }
    SearchSridDialog (MyFrame * parent);
    bool Create (MyFrame * parent);
    virtual ~ SearchSridDialog () { ; }
    void CreateControls ();
    wxString & GetString () { return String; }
    void OnOk (wxCommandEvent & event);
};

class SetSridDialog:public wxDialog
{
//
// a dialog preparing a SET SRID
//
  private:
    MyFrame * MainFrame;
    wxString Table;		// the table's name 
    wxString Column;		// the column's name to be recovered
    int OldSrid;		// SRID to substitute
    int Srid;			// required SRID
  public:
      SetSridDialog () { ; }
    SetSridDialog (MyFrame * parent, wxString & table, wxString & column);
    bool Create (MyFrame * parent, wxString & table, wxString & column);
    virtual ~ SetSridDialog () { ; }
    void CreateControls ();
    int GetOldSrid () { return OldSrid; }
    int GetSrid () { return Srid; }
    void OnOk (wxCommandEvent & event);
};

class RecoverDialog:public wxDialog
{
//
// a dialog preparing a RECOVER GEOMETRY
//
  private:
    MyFrame * MainFrame;
    wxString Table;		// the table's name 
    wxString Column;		// the column's name to be recovered
    int Srid;			// required SRID
    wxString Type;		// required Geometry Type
  public:
      RecoverDialog () { ; }
    RecoverDialog (MyFrame * parent, wxString & table, wxString & column);
    bool Create (MyFrame * parent, wxString & table, wxString & column);
    virtual ~ RecoverDialog () { ; }
    void CreateControls ();
    wxString & GetType () { return Type; }
    int GetSrid () { return Srid; }
    void OnOk (wxCommandEvent & event);
};

class VirtualShpDialog:public wxDialog
{
//
// a dialog preparing a CREATE VIRTUAL SHAPE
//
  private:
    MyFrame * MainFrame;
    wxString Path;		// the SHP base path
	wxString Table;		// the table name
    wxString Default;		// the default charset
    wxString Charset;		// the SHP charset
    int Srid;			// the SRID
  public:
      VirtualShpDialog () { ; }
    VirtualShpDialog (MyFrame * parent, wxString & path, wxString & table, wxString & defCs);
    bool Create (MyFrame * parent, wxString & path, wxString & table, wxString & defCs);
    virtual ~ VirtualShpDialog () { ; }
    void CreateControls ();
	wxString & GetTable() { return Table; }
    wxString & GetCharset () { 	return Charset; }
    int GetSrid () { return Srid; }
    void OnOk (wxCommandEvent & event);
};

class VirtualTxtDialog:public wxDialog
{
//
// a dialog preparing a CREATE VIRTUAL TEXT
//
  private:
    MyFrame * MainFrame;
    wxString Path;		// the CSV/TXT base path
	wxString Table;		// the table name
    wxString Default;		// the default charset
    wxString Charset;		// the CSV/TXT charset
	bool FirstLineTitles;	// TRUE if first line stores column titles
	char Separator;			// the character to be used as field separator
	char TextSeparator;		// the character to be used as text separator
	bool DecimalPointIsComma;		// TRUE if decimal separator is COMMA
  public:
      VirtualTxtDialog () { ; }
    VirtualTxtDialog (MyFrame * parent, wxString & path, wxString & table, wxString & defCs);
    bool Create (MyFrame * parent, wxString & path, wxString & table, wxString & defCs);
    virtual ~ VirtualTxtDialog () { ; }
    void CreateControls ();
	wxString & GetTable() { return Table; }
    wxString & GetCharset () { 	return Charset; }
	bool IsFirstLineTitles() { return FirstLineTitles; }
	char GetSeparator() { return Separator; }
	char GetTextSeparator() { return TextSeparator; }
	bool IsDecimalPointComma() { return DecimalPointIsComma; }
	void OnSeparator (wxCommandEvent & event);
	void OnDecimalSeparator (wxCommandEvent & event);
	void OnQuote (wxCommandEvent & event);
    void OnOk (wxCommandEvent & event);
};

class LoadShpDialog:public wxDialog
{
//
// a dialog preparing a LOAD SHAPE
//
  private:
    MyFrame * MainFrame;
    wxString Path;		// the SHP base path
    wxString Table;		// the table's name to be created
    wxString Column;		// the column's name for Geometry
    wxString Default;		// the default charset
    wxString Charset;		// the SHP charset
    int Srid;			// the SRID
  public:
      LoadShpDialog () { ; }
    LoadShpDialog (MyFrame * parent, wxString & path, wxString & table,
		   int srid, wxString & column, wxString & defCs);
    bool Create (MyFrame * parent, wxString & path, wxString & table, int srid,
		 wxString & column, wxString & defCs);
    virtual ~ LoadShpDialog () { ; }
    void CreateControls ();
    wxString & GetTable () { return Table; }
    wxString & GetColumn () { return Column; }
    wxString & GetCharset () { return Charset; }
    int GetSrid () { return Srid; }
    void OnOk (wxCommandEvent & event);
};

class DumpShpDialog:public wxDialog
{
//
// a dialog preparing a DUMP SHAPE
//
  private:
    MyFrame * MainFrame;
    wxString Path;		// the SHP base path
    wxString Table;		// the table's name to be created
    wxString Column;		// the column's name for Geometry
    wxString Default;		// the default charset
    wxString Charset;		// the SHP charset
  public:
      DumpShpDialog () { ; }
    DumpShpDialog (MyFrame * parent, wxString & path, wxString & table,
		   wxString & column, wxString & defCs);
    bool Create (MyFrame * parent, wxString & path, wxString & table,
		 wxString & column, wxString & defCs);
    virtual ~ DumpShpDialog () { ; }
    void CreateControls ();
    wxString & GetCharset () { return Charset; }
    void OnOk (wxCommandEvent & event);
};

class LoadTxtDialog:public wxDialog
{
//
// a dialog preparing a LOAD TXT/CSV
//
  private:
    MyFrame * MainFrame;
    wxString Path;		// the CSV/TXT base path
	wxString Table;		// the table name
    wxString Default;		// the default charset
    wxString Charset;		// the CSV/TXT charset
	bool FirstLineTitles;	// TRUE if first line stores column titles
	char Separator;			// the character to be used as field separator
	char TextSeparator;		// the character to be used as text separator
	bool DecimalPointIsComma;		// TRUE if decimal separator is COMMA
  public:
      LoadTxtDialog () { ; }
    LoadTxtDialog (MyFrame * parent, wxString & path, wxString & table, wxString & defCs);
    bool Create (MyFrame * parent, wxString & path, wxString & table, wxString & defCs);
    virtual ~ LoadTxtDialog () { ; }
    void CreateControls ();
	wxString & GetTable() { return Table; }
    wxString & GetCharset () { 	return Charset; }
	bool IsFirstLineTitles() { return FirstLineTitles; }
	char GetSeparator() { return Separator; }
	char GetTextSeparator() { return TextSeparator; }
	bool IsDecimalPointComma() { return DecimalPointIsComma; }
	void OnSeparator (wxCommandEvent & event);
	void OnDecimalSeparator (wxCommandEvent & event);
	void OnQuote (wxCommandEvent & event);
    void OnOk (wxCommandEvent & event);
};

class DumpTxtDialog:public wxDialog
{
//
// a dialog preparing a DUMP generic text
//
  private:
    MyFrame * MainFrame;
    wxString Path;		// the SHP base path
    wxString Default;		// the default charset
    wxString Charset;		// the target charset
  public:
      DumpTxtDialog () { ; }
    DumpTxtDialog (MyFrame * parent, wxString & path, wxString & target,
		   wxString & defCs);
    bool Create (MyFrame * parent, wxString & path, wxString & target,
		 wxString & defCs);
    virtual ~ DumpTxtDialog () { ; }
    void CreateControls ();
    wxString & GetCharset () { return Charset; }
    void OnOk (wxCommandEvent & event);
};

class SqlScriptDialog:public wxDialog
{
//
// a dialog preparing an SQL SCRIPT execute
//
  private:
    MyFrame * MainFrame;
    wxString Path;		// the SHP base path
    wxString Default;		// the default charset
    wxString Charset;		// the target charset
  public:
      SqlScriptDialog () { ; }
    SqlScriptDialog (MyFrame * parent, wxString & path, wxString & defCs);
    bool Create (MyFrame * parent, wxString & path, wxString & defCs);
    virtual ~ SqlScriptDialog () { ; }
    void CreateControls ();
    wxString & GetCharset () { return Charset; }
    void OnOk (wxCommandEvent & event);
};

class DefaultCharsetDialog:public wxDialog
{
//
// a dialog for selecting DEFAULT CHARSET
//
  private:
    MyFrame * MainFrame;
    wxString Charset;		// the default charset
    bool AskCharset;		// true / false
  public:
      DefaultCharsetDialog () { ; }
    DefaultCharsetDialog (MyFrame * parent, wxString & charset, bool ask);
    bool Create (MyFrame * parent, wxString & charset, bool ask);
    virtual ~ DefaultCharsetDialog () { ; }
    void CreateControls ();
    wxString & GetCharset () { return Charset; }
    bool IsSetAskCharset () { return AskCharset; }
    void OnOk (wxCommandEvent & event);
};


class BlobExplorerDialog:public wxPropertySheetDialog
{
//
// a dialog to explore a BLOB value
//
  private:
    MyFrame * MainFrame;
    int BlobSize;		// the BLOB size
    unsigned char *Blob;	// the BLOB value
    int BlobType;		// the BLOB type
    gaiaGeomCollPtr Geometry;	// the geometry [optional]
    wxImage *Image;		// the image [optional]
    wxBitmap GeomPreview;	// the geometry preview
  public:
      BlobExplorerDialog ()
    {;
    }
    BlobExplorerDialog (MyFrame * parent, int blob_size, unsigned char *blob);
    bool Create (MyFrame * parent, int blob_size, unsigned char *blob);
    virtual ~ BlobExplorerDialog () { if (Geometry) gaiaFreeGeomColl (Geometry);
		if (Image) delete Image; }
    void DrawGeometry (int horz, int vert);
    wxPanel *CreateHexadecimalPage (wxWindow * book);
    wxPanel *CreateGeometryPage (wxWindow * book);
    wxPanel *CreateImagePage (wxWindow * book);
    void UpdateHexadecimalPage ();
    void UpdateGeometryPage ();
    void UpdateImagePage ();
    gaiaGeomCollPtr GetGeometry () { return Geometry; }
    wxImage *GetImage () { return Image; }
    int GetBlobType () { return BlobType; }
    void OnOk (wxCommandEvent & event);
    void OnPageChanged (wxNotebookEvent & event);
};

class GraphicsGeometry:public wxStaticBitmap
{
//
// a window to show some Geometry in a graphical fashion
//
  private:
    BlobExplorerDialog * Parent;
  public:
    GraphicsGeometry (BlobExplorerDialog * parent, wxWindow * panel,
		      wxWindowID id, const wxBitmap & bmp, wxSize const &size);
      virtual ~ GraphicsGeometry () { ; }
};

class ImageShow:public wxStaticBitmap
{
//
// a window to show some Image [Jpeg-Png-Gif]
//
  private:
    BlobExplorerDialog * Parent;
  public:
    ImageShow (BlobExplorerDialog * parent, wxWindow * panel, wxWindowID id,
	       const wxBitmap & bmp, const wxSize & size);
      virtual ~ ImageShow () { ; }
    void OnRightClick (wxMouseEvent & event);
    void OnCmdCopy (wxCommandEvent & event);
};

class MyHexList:public wxListCtrl
{
//
// a class for Hexdecimal dumps
//
  private:
    BlobExplorerDialog * Parent;
    int BlobSize;		// the BLOB size
    unsigned char *Blob;	// the BLOB value
  public:
      MyHexList (BlobExplorerDialog * parent, unsigned char *blob,
		 int blob_size, wxWindow * panel, wxWindowID id,
		 const wxPoint & pos = wxDefaultPosition, const wxSize & size =
		 wxDefaultSize, long style = 0);
      virtual ~ MyHexList ();
    virtual wxString OnGetItemText (long item, long column) const;
};
