/*
/ Dialogs.cpp
/ various dialog classes
/
/ version 1.1, 2008 September 13
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2008  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU General Public License for more details.
/
/    You should have received a copy of the GNU General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include "Classdef.h"

#include "wx/spinctrl.h"
#include "wx/listctrl.h"
#include "wx/html/htmlwin.h"

bool
VirtualShpDialog::Create (MyFrame * parent, wxString & path, wxString & table,
			  wxString & defCs)
{
//
// creating the dialog
//
    MainFrame = parent;
    Path = path;
    Table = table;
    Default = defCs;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Creating Virtual Shapefile"))
	== false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
VirtualShpDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// first row: the Shapefile path
    wxBoxSizer *pathSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (pathSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *pathLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Path:"));
    pathSizer->Add (pathLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *pathValue = new wxTextCtrl (this, wxID_STATIC,
					    Path, wxDefaultPosition,
					    wxSize (350, 22), wxTE_READONLY);
    pathSizer->Add (pathValue, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: TABLE name
    wxBoxSizer *tableSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (tableSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *tableLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Table name:"));
    tableSizer->Add (tableLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *tableCtrl = new wxTextCtrl (this, ID_VIRTSHP_TABLE, Table,
					    wxDefaultPosition, wxSize (350,
								       22));
    tableSizer->Add (tableCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: SRID
    wxBoxSizer *sridSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (sridSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxStaticText *sridLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&SRID:"));
    sridSizer->Add (sridLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxSpinCtrl *sridCtrl = new wxSpinCtrl (this, ID_VIRTSHP_SRID, wxEmptyString,
					   wxDefaultPosition, wxSize (80, 20),
					   wxSP_ARROW_KEYS,
					   -1, 40000, -1);
    sridSizer->Add (sridCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// third row: CHARSET
    wxStaticBox *charsetBox = new wxStaticBox (this, wxID_STATIC,
					       wxT ("Charset Encoding"),
					       wxDefaultPosition,
					       wxDefaultSize);
    wxBoxSizer *charsetSizer = new wxStaticBoxSizer (charsetBox, wxHORIZONTAL);
    sridSizer->Add (charsetSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxListBox *charsetCtrl = new wxListBox (this, ID_VIRTSHP_CHARSET,
					    wxDefaultPosition, wxDefaultSize,
					    MainFrame->GetCharsetsLen (),
					    MainFrame->GetCharsetsNames (),
					    wxLB_SINGLE);
    charsetCtrl->SetFont (wxFont
			  (8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL,
			   wxFONTWEIGHT_NORMAL));
    int idSel = MainFrame->GetCharsetIndex (Default);
    if (idSel != wxNOT_FOUND)
	charsetCtrl->SetSelection (idSel);
    charsetSizer->Add (charsetCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & VirtualShpDialog::OnOk);
}

void
VirtualShpDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxTextCtrl *tableCtrl = (wxTextCtrl *) FindWindow (ID_VIRTSHP_TABLE);
    Table = tableCtrl->GetValue ();
    if (Table.Len () < 1)
      {
	  wxMessageBox (wxT ("You must specify the TABLE NAME !!!"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    if (MainFrame->TableAlreadyExists (Table) == true)
      {
	  wxMessageBox (wxT ("a table name '") + Table +
			wxT ("' already exists"), wxT ("spatialite-gui"),
			wxOK | wxICON_WARNING, this);
	  return;
      }
    if (gaiaIllegalSqlName (Table.ToUTF8 ()) == 1
	|| gaiaIsReservedSqlName (Table.ToUTF8 ()) == 1
	|| gaiaIsReservedSqliteName (Table.ToUTF8 ()) == 1)
      {
	  wxMessageBox (wxT ("'") + Table +
			wxT
			("' is an invalid TABLE NAME\n\nillegal name or reserved keyword"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxSpinCtrl *sridCtrl = (wxSpinCtrl *) FindWindow (ID_VIRTSHP_SRID);
    Srid = sridCtrl->GetValue ();
    if (Srid == -1)
	;
    else if (MainFrame->SridNotExists (Srid) == true)
      {
	  wxMessageBox (wxT ("invalid SRID value"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxListBox *charsetCtrl = (wxListBox *) FindWindow (ID_VIRTSHP_CHARSET);
    int idSel = charsetCtrl->GetSelection ();
    if (idSel == wxNOT_FOUND)
      {
	  wxMessageBox (wxT
			("you must select some Charset Encoding from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxString *charsets = MainFrame->GetCharsets ();
    Charset = *(charsets + idSel);
    wxDialog::EndModal (wxID_OK);
}

bool
VirtualTxtDialog::Create (MyFrame * parent, wxString & path, wxString & table,
			  wxString & defCs)
{
//
// creating the dialog
//
    MainFrame = parent;
    Path = path;
    Table = table;
    Default = defCs;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Creating Virtual CSV/TXT"))
	== false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
VirtualTxtDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    FirstLineTitles = true;
    Separator = '\t';
    TextSeparator = '"';
    DecimalPointIsComma = false;
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// first row: the CSV/TXT path
    wxBoxSizer *pathSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (pathSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *pathLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Path:"));
    pathSizer->Add (pathLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *pathValue = new wxTextCtrl (this, wxID_STATIC,
					    Path, wxDefaultPosition,
					    wxSize (350, 22), wxTE_READONLY);
    pathSizer->Add (pathValue, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: TABLE name
    wxBoxSizer *tableSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (tableSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *tableLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Table name:"));
    tableSizer->Add (tableLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *tableCtrl = new wxTextCtrl (this, ID_VIRTTXT_TABLE, Table,
					    wxDefaultPosition, wxSize (350,
								       22));
    tableSizer->Add (tableCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: OPTIONS
    wxBoxSizer *optSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (optSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxBoxSizer *modeSizer = new wxBoxSizer (wxVERTICAL);
    optSizer->Add (modeSizer, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
    wxBoxSizer *mode1Sizer = new wxBoxSizer (wxHORIZONTAL);
    modeSizer->Add (mode1Sizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxCheckBox *titleCtrl = new wxCheckBox (this, ID_VIRTTXT_TITLES,
					    wxT
					    ("First line contains column names"),
					    wxDefaultPosition, wxDefaultSize);
    titleCtrl->SetValue (true);
    mode1Sizer->Add (titleCtrl, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxStaticBox *sepBox = new wxStaticBox (this, wxID_STATIC,
					   wxT ("Separators"),
					   wxDefaultPosition, wxDefaultSize);
    wxBoxSizer *sepSizer = new wxStaticBoxSizer (sepBox, wxVERTICAL);
    modeSizer->Add (sepSizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxBoxSizer *mode2Sizer = new wxBoxSizer (wxHORIZONTAL);
    sepSizer->Add (mode2Sizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxString quotes[2];
    quotes[0] = wxT ("&Double \"");
    quotes[1] = wxT ("&Single '");
    wxRadioBox *textSeparatorBox = new wxRadioBox (this, ID_VIRTTXT_QUOTE,
						   wxT
						   ("&Text separator: quotes"),
						   wxDefaultPosition,
						   wxDefaultSize, 2,
						   quotes, 2,
						   wxRA_SPECIFY_COLS);
    mode2Sizer->Add (textSeparatorBox, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxBoxSizer *mode3Sizer = new wxBoxSizer (wxHORIZONTAL);
    sepSizer->Add (mode3Sizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxString separators[6];
    separators[0] = wxT ("&Tab");
    separators[1] = wxT ("&Space");
    separators[2] = wxT ("Co&mma ,");
    separators[3] = wxT ("&Colon :");
    separators[4] = wxT ("S&emicolon ;");
    separators[5] = wxT ("&Other");
    wxRadioBox *separatorBox = new wxRadioBox (this, ID_VIRTTXT_SEPARATOR,
					       wxT ("&Column separator"),
					       wxDefaultPosition, wxDefaultSize,
					       6,
					       separators, 2,
					       wxRA_SPECIFY_COLS);
    mode3Sizer->Add (separatorBox, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxBoxSizer *mode4Sizer = new wxBoxSizer (wxHORIZONTAL);
    sepSizer->Add (mode4Sizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxStaticText *charSeparatorLabel = new wxStaticText (this, wxID_STATIC,
							 wxT
							 ("&Custom separator:"),
							 wxDefaultPosition,
							 wxDefaultSize, 0);
    mode4Sizer->Add (charSeparatorLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *dummy =
	new wxTextCtrl (this, ID_VIRTTXT_CHARSEPARATOR, wxT ("Dummy Text pq"),
			wxDefaultPosition,
			wxDefaultSize, 0);
    wxSize sz = dummy->GetSize ();
    delete dummy;
    sz.SetWidth (40);
    wxTextCtrl *charSeparatorCtrl =
	new wxTextCtrl (this, ID_VIRTTXT_CHARSEPARATOR, wxT ("TAB"),
			wxDefaultPosition, sz, 0);
    charSeparatorCtrl->SetMaxLength (1);
    charSeparatorCtrl->Enable (false);
    mode4Sizer->Add (charSeparatorCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxBoxSizer *mode5Sizer = new wxBoxSizer (wxHORIZONTAL);
    sepSizer->Add (mode5Sizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxString points[2];
    points[0] = wxT ("&Point .");
    points[1] = wxT ("&Comma ,");
    wxRadioBox *decimalSeparatorBox = new wxRadioBox (this, ID_VIRTTXT_POINT,
						      wxT
						      ("&Decimal separator"),
						      wxDefaultPosition,
						      wxDefaultSize, 2,
						      points, 2,
						      wxRA_SPECIFY_COLS);
    mode5Sizer->Add (decimalSeparatorBox, 0, wxALIGN_CENTER_VERTICAL | wxALL,
		     5);
// third row: CHARSET
    wxStaticBox *charsetBox = new wxStaticBox (this, wxID_STATIC,
					       wxT ("Charset Encoding"),
					       wxDefaultPosition,
					       wxDefaultSize);
    wxBoxSizer *charsetSizer = new wxStaticBoxSizer (charsetBox, wxHORIZONTAL);
    optSizer->Add (charsetSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxListBox *charsetCtrl = new wxListBox (this, ID_VIRTTXT_CHARSET,
					    wxDefaultPosition, wxDefaultSize,
					    MainFrame->GetCharsetsLen (),
					    MainFrame->GetCharsetsNames (),
					    wxLB_SINGLE);
    charsetCtrl->SetFont (wxFont
			  (8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL,
			   wxFONTWEIGHT_NORMAL));
    int idSel = MainFrame->GetCharsetIndex (Default);
    if (idSel != wxNOT_FOUND)
	charsetCtrl->SetSelection (idSel);
    charsetSizer->Add (charsetCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & VirtualTxtDialog::OnOk);
// appends event handler for radioboxes
    Connect (ID_VIRTTXT_QUOTE, wxEVT_COMMAND_RADIOBOX_SELECTED,
	     (wxObjectEventFunction) & VirtualTxtDialog::OnQuote);
    Connect (ID_VIRTTXT_SEPARATOR, wxEVT_COMMAND_RADIOBOX_SELECTED,
	     (wxObjectEventFunction) & VirtualTxtDialog::OnSeparator);
    Connect (ID_VIRTTXT_POINT, wxEVT_COMMAND_RADIOBOX_SELECTED,
	     (wxObjectEventFunction) & VirtualTxtDialog::OnDecimalSeparator);
}

void
VirtualTxtDialog::OnDecimalSeparator (wxCommandEvent & event)
{
//
// POINT selection changed
//
    wxRadioBox *decimalSeparatorCtrl =
	(wxRadioBox *) FindWindow (ID_VIRTTXT_POINT);
    switch (decimalSeparatorCtrl->GetSelection ())
      {
      case 0:
	  DecimalPointIsComma = false;
	  break;
      case 1:
	  DecimalPointIsComma = true;
	  break;
      };
}

void
VirtualTxtDialog::OnQuote (wxCommandEvent & event)
{
//
// QUOTE selection changed
//
    wxRadioBox *separatorCtrl = (wxRadioBox *) FindWindow (ID_VIRTTXT_QUOTE);
    switch (separatorCtrl->GetSelection ())
      {
      case 0:
	  TextSeparator = '"';
	  break;
      case 1:
	  TextSeparator = '\'';
	  break;
      };
}

void
VirtualTxtDialog::OnSeparator (wxCommandEvent & event)
{
//
// SEPARATOR selection changed
//
    wxTextCtrl *charSeparatorCtrl =
	(wxTextCtrl *) FindWindow (ID_VIRTTXT_CHARSEPARATOR);
    wxRadioBox *separatorCtrl =
	(wxRadioBox *) FindWindow (ID_VIRTTXT_SEPARATOR);
    switch (separatorCtrl->GetSelection ())
      {
      case 0:
	  Separator = '\t';
	  charSeparatorCtrl->SetValue (wxT ("TAB"));
	  break;
      case 1:
	  Separator = ' ';
	  charSeparatorCtrl->SetValue (wxT ("SP"));
	  break;
      case 2:
	  Separator = ',';
	  charSeparatorCtrl->SetValue (wxT (","));
	  break;
      case 3:
	  Separator = ':';
	  charSeparatorCtrl->SetValue (wxT (":"));
	  break;
      case 4:
	  Separator = ';';
	  charSeparatorCtrl->SetValue (wxT (";"));
	  break;
      case 5:
	  Separator = '|';
	  charSeparatorCtrl->SetValue (wxT ("|"));
	  break;
      };
    if (separatorCtrl->GetSelection () == 5)
	charSeparatorCtrl->Enable (true);
    else
	charSeparatorCtrl->Enable (false);
}

void
VirtualTxtDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxTextCtrl *tableCtrl = (wxTextCtrl *) FindWindow (ID_VIRTTXT_TABLE);
    Table = tableCtrl->GetValue ();
    if (Table.Len () < 1)
      {
	  wxMessageBox (wxT ("You must specify the TABLE NAME !!!"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    if (MainFrame->TableAlreadyExists (Table) == true)
      {
	  wxMessageBox (wxT ("a table name '") + Table +
			wxT ("' already exists"), wxT ("spatialite-gui"),
			wxOK | wxICON_WARNING, this);
	  return;
      }
    if (gaiaIllegalSqlName (Table.ToUTF8 ()) == 1
	|| gaiaIsReservedSqlName (Table.ToUTF8 ()) == 1
	|| gaiaIsReservedSqliteName (Table.ToUTF8 ()) == 1)
      {
	  wxMessageBox (wxT ("'") + Table +
			wxT
			("' is an invalid TABLE NAME\n\nillegal name or reserved keyword"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxListBox *charsetCtrl = (wxListBox *) FindWindow (ID_VIRTTXT_CHARSET);
    int idSel = charsetCtrl->GetSelection ();
    if (idSel == wxNOT_FOUND)
      {
	  wxMessageBox (wxT
			("you must select some Charset Encoding from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxString *charsets = MainFrame->GetCharsets ();
    Charset = *(charsets + idSel);
    wxRadioBox *separatorCtrl =
	(wxRadioBox *) FindWindow (ID_VIRTTXT_SEPARATOR);
    if (separatorCtrl->GetSelection () == 5)
      {
	  wxTextCtrl *charSeparatorCtrl =
	      (wxTextCtrl *) FindWindow (ID_VIRTTXT_CHARSEPARATOR);
	  wxString separator = charSeparatorCtrl->GetValue ();
	  if (separator.Len () != 1)
	    {
		wxMessageBox (wxT
			      ("you must specificy a single char as Custom Column Separator"),
			      wxT ("spatialite-gui"), wxOK | wxICON_WARNING,
			      this);
		return;
	    }
	  char dummy[64];
	  strcpy (dummy, separator.ToUTF8 ());
	  Separator = *dummy;
      }
    wxDialog::EndModal (wxID_OK);
}

bool
LoadTxtDialog::Create (MyFrame * parent, wxString & path, wxString & table,
		       wxString & defCs)
{
//
// creating the dialog
//
    MainFrame = parent;
    Path = path;
    Table = table;
    Default = defCs;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Load CSV/TXT")) == false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
LoadTxtDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    FirstLineTitles = true;
    Separator = '\t';
    TextSeparator = '"';
    DecimalPointIsComma = false;
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// first row: the CSV/TXT path
    wxBoxSizer *pathSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (pathSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *pathLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Path:"));
    pathSizer->Add (pathLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *pathValue = new wxTextCtrl (this, wxID_STATIC,
					    Path, wxDefaultPosition,
					    wxSize (350, 22), wxTE_READONLY);
    pathSizer->Add (pathValue, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: TABLE name
    wxBoxSizer *tableSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (tableSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *tableLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Table name:"));
    tableSizer->Add (tableLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *tableCtrl = new wxTextCtrl (this, ID_LDTXT_TABLE, Table,
					    wxDefaultPosition, wxSize (350,
								       22));
    tableSizer->Add (tableCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: OPTIONS
    wxBoxSizer *optSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (optSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxBoxSizer *modeSizer = new wxBoxSizer (wxVERTICAL);
    optSizer->Add (modeSizer, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
    wxBoxSizer *mode1Sizer = new wxBoxSizer (wxHORIZONTAL);
    modeSizer->Add (mode1Sizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxCheckBox *titleCtrl = new wxCheckBox (this, ID_LDTXT_TITLES,
					    wxT
					    ("First line contains column names"),
					    wxDefaultPosition, wxDefaultSize);
    titleCtrl->SetValue (true);
    mode1Sizer->Add (titleCtrl, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxStaticBox *sepBox = new wxStaticBox (this, wxID_STATIC,
					   wxT ("Separators"),
					   wxDefaultPosition, wxDefaultSize);
    wxBoxSizer *sepSizer = new wxStaticBoxSizer (sepBox, wxVERTICAL);
    modeSizer->Add (sepSizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxBoxSizer *mode2Sizer = new wxBoxSizer (wxHORIZONTAL);
    sepSizer->Add (mode2Sizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxString quotes[2];
    quotes[0] = wxT ("&Double \"");
    quotes[1] = wxT ("&Single '");
    wxRadioBox *textSeparatorBox = new wxRadioBox (this, ID_LDTXT_QUOTE,
						   wxT
						   ("&Text separator: quotes"),
						   wxDefaultPosition,
						   wxDefaultSize, 2,
						   quotes, 2,
						   wxRA_SPECIFY_COLS);
    mode2Sizer->Add (textSeparatorBox, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxBoxSizer *mode3Sizer = new wxBoxSizer (wxHORIZONTAL);
    sepSizer->Add (mode3Sizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxString separators[6];
    separators[0] = wxT ("&Tab");
    separators[1] = wxT ("&Space");
    separators[2] = wxT ("Co&mma ,");
    separators[3] = wxT ("&Colon :");
    separators[4] = wxT ("S&emicolon ;");
    separators[5] = wxT ("&Other");
    wxRadioBox *separatorBox = new wxRadioBox (this, ID_LDTXT_SEPARATOR,
					       wxT ("&Column separator"),
					       wxDefaultPosition, wxDefaultSize,
					       6,
					       separators, 2,
					       wxRA_SPECIFY_COLS);
    mode3Sizer->Add (separatorBox, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxBoxSizer *mode4Sizer = new wxBoxSizer (wxHORIZONTAL);
    sepSizer->Add (mode4Sizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxStaticText *charSeparatorLabel = new wxStaticText (this, wxID_STATIC,
							 wxT
							 ("&Custom separator:"),
							 wxDefaultPosition,
							 wxDefaultSize, 0);
    mode4Sizer->Add (charSeparatorLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *dummy =
	new wxTextCtrl (this, ID_LDTXT_CHARSEPARATOR, wxT ("Dummy Text pq"),
			wxDefaultPosition,
			wxDefaultSize, 0);
    wxSize sz = dummy->GetSize ();
    delete dummy;
    sz.SetWidth (40);
    wxTextCtrl *charSeparatorCtrl =
	new wxTextCtrl (this, ID_LDTXT_CHARSEPARATOR, wxT ("TAB"),
			wxDefaultPosition, sz, 0);
    charSeparatorCtrl->SetMaxLength (1);
    charSeparatorCtrl->Enable (false);
    mode4Sizer->Add (charSeparatorCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxBoxSizer *mode5Sizer = new wxBoxSizer (wxHORIZONTAL);
    sepSizer->Add (mode5Sizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxString points[2];
    points[0] = wxT ("&Point .");
    points[1] = wxT ("&Comma ,");
    wxRadioBox *decimalSeparatorBox = new wxRadioBox (this, ID_LDTXT_POINT,
						      wxT
						      ("&Decimal separator"),
						      wxDefaultPosition,
						      wxDefaultSize, 2,
						      points, 2,
						      wxRA_SPECIFY_COLS);
    mode5Sizer->Add (decimalSeparatorBox, 0, wxALIGN_CENTER_VERTICAL | wxALL,
		     5);
// third row: CHARSET
    wxStaticBox *charsetBox = new wxStaticBox (this, wxID_STATIC,
					       wxT ("Charset Encoding"),
					       wxDefaultPosition,
					       wxDefaultSize);
    wxBoxSizer *charsetSizer = new wxStaticBoxSizer (charsetBox, wxHORIZONTAL);
    optSizer->Add (charsetSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxListBox *charsetCtrl = new wxListBox (this, ID_LDTXT_CHARSET,
					    wxDefaultPosition, wxDefaultSize,
					    MainFrame->GetCharsetsLen (),
					    MainFrame->GetCharsetsNames (),
					    wxLB_SINGLE);
    charsetCtrl->SetFont (wxFont
			  (8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL,
			   wxFONTWEIGHT_NORMAL));
    int idSel = MainFrame->GetCharsetIndex (Default);
    if (idSel != wxNOT_FOUND)
	charsetCtrl->SetSelection (idSel);
    charsetSizer->Add (charsetCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & LoadTxtDialog::OnOk);
// appends event handler for radioboxes
    Connect (ID_LDTXT_QUOTE, wxEVT_COMMAND_RADIOBOX_SELECTED,
	     (wxObjectEventFunction) & LoadTxtDialog::OnQuote);
    Connect (ID_LDTXT_SEPARATOR, wxEVT_COMMAND_RADIOBOX_SELECTED,
	     (wxObjectEventFunction) & LoadTxtDialog::OnSeparator);
    Connect (ID_LDTXT_POINT, wxEVT_COMMAND_RADIOBOX_SELECTED,
	     (wxObjectEventFunction) & LoadTxtDialog::OnDecimalSeparator);
}

void
LoadTxtDialog::OnDecimalSeparator (wxCommandEvent & event)
{
//
// POINT selection changed
//
    wxRadioBox *decimalSeparatorCtrl =
	(wxRadioBox *) FindWindow (ID_LDTXT_POINT);
    switch (decimalSeparatorCtrl->GetSelection ())
      {
      case 0:
	  DecimalPointIsComma = false;
	  break;
      case 1:
	  DecimalPointIsComma = true;
	  break;
      };
}

void
LoadTxtDialog::OnQuote (wxCommandEvent & event)
{
//
// QUOTE selection changed
//
    wxRadioBox *separatorCtrl = (wxRadioBox *) FindWindow (ID_LDTXT_QUOTE);
    switch (separatorCtrl->GetSelection ())
      {
      case 0:
	  TextSeparator = '"';
	  break;
      case 1:
	  TextSeparator = '\'';
	  break;
      };
}

void
LoadTxtDialog::OnSeparator (wxCommandEvent & event)
{
//
// SEPARATOR selection changed
//
    wxTextCtrl *charSeparatorCtrl =
	(wxTextCtrl *) FindWindow (ID_LDTXT_CHARSEPARATOR);
    wxRadioBox *separatorCtrl = (wxRadioBox *) FindWindow (ID_LDTXT_SEPARATOR);
    switch (separatorCtrl->GetSelection ())
      {
      case 0:
	  Separator = '\t';
	  charSeparatorCtrl->SetValue (wxT ("TAB"));
	  break;
      case 1:
	  Separator = ' ';
	  charSeparatorCtrl->SetValue (wxT ("SP"));
	  break;
      case 2:
	  Separator = ',';
	  charSeparatorCtrl->SetValue (wxT (","));
	  break;
      case 3:
	  Separator = ':';
	  charSeparatorCtrl->SetValue (wxT (":"));
	  break;
      case 4:
	  Separator = ';';
	  charSeparatorCtrl->SetValue (wxT (";"));
	  break;
      case 5:
	  Separator = '|';
	  charSeparatorCtrl->SetValue (wxT ("|"));
	  break;
      };
    if (separatorCtrl->GetSelection () == 5)
	charSeparatorCtrl->Enable (true);
    else
	charSeparatorCtrl->Enable (false);
}

void
LoadTxtDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxTextCtrl *tableCtrl = (wxTextCtrl *) FindWindow (ID_LDTXT_TABLE);
    Table = tableCtrl->GetValue ();
    if (Table.Len () < 1)
      {
	  wxMessageBox (wxT ("You must specify the TABLE NAME !!!"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    if (MainFrame->TableAlreadyExists (Table) == true)
      {
	  wxMessageBox (wxT ("a table name '") + Table +
			wxT ("' already exists"), wxT ("spatialite-gui"),
			wxOK | wxICON_WARNING, this);
	  return;
      }
    if (gaiaIllegalSqlName (Table.ToUTF8 ()) == 1
	|| gaiaIsReservedSqlName (Table.ToUTF8 ()) == 1
	|| gaiaIsReservedSqliteName (Table.ToUTF8 ()) == 1)
      {
	  wxMessageBox (wxT ("'") + Table +
			wxT
			("' is an invalid TABLE NAME\n\nillegal name or reserved keyword"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxListBox *charsetCtrl = (wxListBox *) FindWindow (ID_LDTXT_CHARSET);
    int idSel = charsetCtrl->GetSelection ();
    if (idSel == wxNOT_FOUND)
      {
	  wxMessageBox (wxT
			("you must select some Charset Encoding from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxString *charsets = MainFrame->GetCharsets ();
    Charset = *(charsets + idSel);
    wxRadioBox *separatorCtrl = (wxRadioBox *) FindWindow (ID_LDTXT_SEPARATOR);
    if (separatorCtrl->GetSelection () == 5)
      {
	  wxTextCtrl *charSeparatorCtrl =
	      (wxTextCtrl *) FindWindow (ID_LDTXT_CHARSEPARATOR);
	  wxString separator = charSeparatorCtrl->GetValue ();
	  if (separator.Len () != 1)
	    {
		wxMessageBox (wxT
			      ("you must specificy a single char as Custom Column Separator"),
			      wxT ("spatialite-gui"), wxOK | wxICON_WARNING,
			      this);
		return;
	    }
	  char dummy[64];
	  strcpy (dummy, separator.ToUTF8 ());
	  Separator = *dummy;
      }
    wxDialog::EndModal (wxID_OK);
}

bool
LoadShpDialog::Create (MyFrame * parent, wxString & path, wxString & table,
		       int srid, wxString & column, wxString & defCs)
{
//
// creating the dialog
//
    MainFrame = parent;
    Path = path;
    Table = table;
    Srid = srid;
    Column = column;
    Default = defCs;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Load Shapefile")) == false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
LoadShpDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// first row: the Shapefile path
    wxBoxSizer *pathSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (pathSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *pathLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Path:"));
    pathSizer->Add (pathLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *pathValue = new wxTextCtrl (this, wxID_STATIC,
					    Path, wxDefaultPosition,
					    wxSize (350, 22), wxTE_READONLY);
    pathSizer->Add (pathValue, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: TABLE name
    wxBoxSizer *tableSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (tableSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *tableLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Table name:"));
    tableSizer->Add (tableLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *tableCtrl = new wxTextCtrl (this, ID_LDSHP_TABLE, Table,
					    wxDefaultPosition, wxSize (350,
								       22));
    tableSizer->Add (tableCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// third row: GEOMETRY COLUMN name
    wxBoxSizer *colSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (colSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *colLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&GeomColumn name:"));
    colSizer->Add (colLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *colCtrl = new wxTextCtrl (this, ID_LDSHP_COLUMN, Column,
					  wxDefaultPosition, wxSize (350, 22));
    colSizer->Add (colCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// fourth row: SRID
    wxBoxSizer *sridSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (sridSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxStaticText *sridLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&SRID:"));
    sridSizer->Add (sridLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxSpinCtrl *sridCtrl = new wxSpinCtrl (this, ID_LDSHP_SRID, wxEmptyString,
					   wxDefaultPosition, wxSize (80, 20),
					   wxSP_ARROW_KEYS,
					   -1, 40000, Srid);
    sridSizer->Add (sridCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// fifth row: CHARSET
    wxStaticBox *charsetBox = new wxStaticBox (this, wxID_STATIC,
					       wxT ("Charset Encoding"),
					       wxDefaultPosition,
					       wxDefaultSize);
    wxBoxSizer *charsetSizer = new wxStaticBoxSizer (charsetBox, wxHORIZONTAL);
    sridSizer->Add (charsetSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxListBox *charsetCtrl = new wxListBox (this, ID_LDSHP_CHARSET,
					    wxDefaultPosition, wxDefaultSize,
					    MainFrame->GetCharsetsLen (),
					    MainFrame->GetCharsetsNames (),
					    wxLB_SINGLE);
    charsetCtrl->SetFont (wxFont
			  (8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL,
			   wxFONTWEIGHT_NORMAL));
    int idSel = MainFrame->GetCharsetIndex (Default);
    if (idSel != wxNOT_FOUND)
	charsetCtrl->SetSelection (idSel);
    charsetSizer->Add (charsetCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & LoadShpDialog::OnOk);
}

void
LoadShpDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxTextCtrl *tableCtrl = (wxTextCtrl *) FindWindow (ID_LDSHP_TABLE);
    Table = tableCtrl->GetValue ();
    if (Table.Len () < 1)
      {
	  wxMessageBox (wxT ("You must specify the TABLE NAME !!!"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    if (MainFrame->TableAlreadyExists (Table) == true)
      {
	  wxMessageBox (wxT ("a table name '") + Table +
			wxT ("' already exists"), wxT ("spatialite-gui"),
			wxOK | wxICON_WARNING, this);
	  return;
      }
    if (gaiaIllegalSqlName (Table.ToUTF8 ()) == 1
	|| gaiaIsReservedSqlName (Table.ToUTF8 ()) == 1
	|| gaiaIsReservedSqliteName (Table.ToUTF8 ()) == 1)
      {
	  wxMessageBox (wxT ("'") + Table +
			wxT
			("' is an invalid TABLE NAME\n\nillegal name or reserved keyword"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxTextCtrl *columnCtrl = (wxTextCtrl *) FindWindow (ID_LDSHP_COLUMN);
    Column = columnCtrl->GetValue ();
    if (Column.Len () < 1)
      {
	  wxMessageBox (wxT ("You must specify the GEOMETRY COLUMN NAME !!!"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    if (gaiaIllegalSqlName (Column.ToUTF8 ()) == 1
	|| gaiaIsReservedSqlName (Column.ToUTF8 ()) == 1
	|| gaiaIsReservedSqliteName (Column.ToUTF8 ()) == 1)
      {
	  wxMessageBox (wxT ("'") + Column +
			wxT
			("' is an invalid COLUMN NAME\n\nsame as SQL reserved keyword"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxSpinCtrl *sridCtrl = (wxSpinCtrl *) FindWindow (ID_LDSHP_SRID);
    Srid = sridCtrl->GetValue ();
    if (MainFrame->SridNotExists (Srid) == true)
      {
	  wxMessageBox (wxT ("invalid SRID value"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxListBox *charsetCtrl = (wxListBox *) FindWindow (ID_LDSHP_CHARSET);
    int idSel = charsetCtrl->GetSelection ();
    if (idSel == wxNOT_FOUND)
      {
	  wxMessageBox (wxT
			("you must select some Charset Encoding from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxString *charsets = MainFrame->GetCharsets ();
    Charset = *(charsets + idSel);
    wxDialog::EndModal (wxID_OK);
}

bool
DumpShpDialog::Create (MyFrame * parent, wxString & path, wxString & table,
		       wxString & column, wxString & defCs)
{
//
// creating the dialog
//
    MainFrame = parent;
    Path = path;
    Table = table;
    Column = column;
    Default = defCs;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Dump Shapefile")) == false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
DumpShpDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// first row: the Shapefile path
    wxBoxSizer *pathSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (pathSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *pathLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Path:"));
    pathSizer->Add (pathLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *pathValue = new wxTextCtrl (this, wxID_STATIC,
					    Path, wxDefaultPosition,
					    wxSize (350, 22), wxTE_READONLY);
    pathSizer->Add (pathValue, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: TABLE name
    wxBoxSizer *tableSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (tableSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *tableLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Table name:"));
    tableSizer->Add (tableLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *tableCtrl = new wxTextCtrl (this, wxID_STATIC, Table,
					    wxDefaultPosition, wxSize (350,
								       22));
    tableCtrl->Enable (false);
    tableSizer->Add (tableCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// third row: GEOMETRY COLUMN name
    wxBoxSizer *colSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (colSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *colLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&GeomColumn name:"));
    colSizer->Add (colLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *colCtrl = new wxTextCtrl (this, wxID_STATIC, Column,
					  wxDefaultPosition, wxSize (350, 22));
    colCtrl->Enable (false);
    colSizer->Add (colCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// fourth row: CHARSET
    wxBoxSizer *csSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (csSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxStaticBox *charsetBox = new wxStaticBox (this, wxID_STATIC,
					       wxT ("Charset Encoding"),
					       wxDefaultPosition,
					       wxDefaultSize);
    wxBoxSizer *charsetSizer = new wxStaticBoxSizer (charsetBox, wxHORIZONTAL);
    csSizer->Add (charsetSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxListBox *charsetCtrl = new wxListBox (this, ID_DMPSHP_CHARSET,
					    wxDefaultPosition, wxDefaultSize,
					    MainFrame->GetCharsetsLen (),
					    MainFrame->GetCharsetsNames (),
					    wxLB_SINGLE);
    charsetCtrl->SetFont (wxFont
			  (8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL,
			   wxFONTWEIGHT_NORMAL));
    int idSel = MainFrame->GetCharsetIndex (Default);
    if (idSel != wxNOT_FOUND)
	charsetCtrl->SetSelection (idSel);
    charsetSizer->Add (charsetCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & DumpShpDialog::OnOk);
}

void
DumpShpDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxListBox *charsetCtrl = (wxListBox *) FindWindow (ID_DMPSHP_CHARSET);
    int idSel = charsetCtrl->GetSelection ();
    if (idSel == wxNOT_FOUND)
      {
	  wxMessageBox (wxT
			("you must select some Charset Encoding from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxString *charsets = MainFrame->GetCharsets ();
    Charset = *(charsets + idSel);
    wxDialog::EndModal (wxID_OK);
}

bool
DumpTxtDialog::Create (MyFrame * parent, wxString & path,
		       wxString & target, wxString & defCs)
{
//
// creating the dialog
//
    MainFrame = parent;
    Path = path;
    Default = defCs;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Dump ") + target) == false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
DumpTxtDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// first row: the Shapefile path
    wxBoxSizer *pathSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (pathSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *pathLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Path:"));
    pathSizer->Add (pathLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *pathValue = new wxTextCtrl (this, wxID_STATIC,
					    Path, wxDefaultPosition,
					    wxSize (350, 22), wxTE_READONLY);
    pathSizer->Add (pathValue, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: CHARSET
    wxBoxSizer *csSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (csSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxStaticBox *charsetBox = new wxStaticBox (this, wxID_STATIC,
					       wxT ("Charset Encoding"),
					       wxDefaultPosition,
					       wxDefaultSize);
    wxBoxSizer *charsetSizer = new wxStaticBoxSizer (charsetBox, wxHORIZONTAL);
    csSizer->Add (charsetSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxListBox *charsetCtrl = new wxListBox (this, ID_DMPTXT_CHARSET,
					    wxDefaultPosition, wxDefaultSize,
					    MainFrame->GetCharsetsLen (),
					    MainFrame->GetCharsetsNames (),
					    wxLB_SINGLE);
    charsetCtrl->SetFont (wxFont
			  (8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL,
			   wxFONTWEIGHT_NORMAL));
    int idSel = MainFrame->GetCharsetIndex (Default);
    if (idSel != wxNOT_FOUND)
	charsetCtrl->SetSelection (idSel);
    charsetSizer->Add (charsetCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & DumpTxtDialog::OnOk);
}

void
DumpTxtDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxListBox *charsetCtrl = (wxListBox *) FindWindow (ID_DMPTXT_CHARSET);
    int idSel = charsetCtrl->GetSelection ();
    if (idSel == wxNOT_FOUND)
      {
	  wxMessageBox (wxT
			("you must select some Charset Encoding from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxString *charsets = MainFrame->GetCharsets ();
    Charset = *(charsets + idSel);
    wxDialog::EndModal (wxID_OK);
}

bool
SqlScriptDialog::Create (MyFrame * parent, wxString & path, wxString & defCs)
{
//
// creating the dialog
//
    MainFrame = parent;
    Path = path;
    Default = defCs;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("SQL script execute")) ==
	false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
SqlScriptDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// first row: the SQL script path
    wxBoxSizer *pathSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (pathSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *pathLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Path:"));
    pathSizer->Add (pathLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *pathValue = new wxTextCtrl (this, wxID_STATIC,
					    Path, wxDefaultPosition,
					    wxSize (350, 22), wxTE_READONLY);
    pathSizer->Add (pathValue, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: CHARSET
    wxBoxSizer *csSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (csSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxStaticBox *charsetBox = new wxStaticBox (this, wxID_STATIC,
					       wxT ("Charset Encoding"),
					       wxDefaultPosition,
					       wxDefaultSize);
    wxBoxSizer *charsetSizer = new wxStaticBoxSizer (charsetBox, wxHORIZONTAL);
    csSizer->Add (charsetSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxListBox *charsetCtrl = new wxListBox (this, ID_SCRIPT_CHARSET,
					    wxDefaultPosition, wxDefaultSize,
					    MainFrame->GetCharsetsLen (),
					    MainFrame->GetCharsetsNames (),
					    wxLB_SINGLE);
    charsetCtrl->SetFont (wxFont
			  (8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL,
			   wxFONTWEIGHT_NORMAL));
    int idSel = MainFrame->GetCharsetIndex (Default);
    if (idSel != wxNOT_FOUND)
	charsetCtrl->SetSelection (idSel);
    charsetSizer->Add (charsetCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & SqlScriptDialog::OnOk);
}

void
SqlScriptDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxListBox *charsetCtrl = (wxListBox *) FindWindow (ID_SCRIPT_CHARSET);
    int idSel = charsetCtrl->GetSelection ();
    if (idSel == wxNOT_FOUND)
      {
	  wxMessageBox (wxT
			("you must select some Charset Encoding from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxString *charsets = MainFrame->GetCharsets ();
    Charset = *(charsets + idSel);
    wxDialog::EndModal (wxID_OK);
}

bool
DefaultCharsetDialog::Create (MyFrame * parent, wxString & charset, bool ask)
{
//
// creating the dialog
//
    MainFrame = parent;
    Charset = charset;
    AskCharset = ask;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Charset settings")) == false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
DefaultCharsetDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// first row: the DEFAULT CHARSET
    wxBoxSizer *csSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (csSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticBox *charsetBox = new wxStaticBox (this, wxID_STATIC,
					       wxT ("Default Output Charset"),
					       wxDefaultPosition,
					       wxDefaultSize);
    wxBoxSizer *charsetSizer = new wxStaticBoxSizer (charsetBox, wxVERTICAL);
    csSizer->Add (charsetSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxListBox *charsetCtrl = new wxListBox (this, ID_DFLT_CHARSET,
					    wxDefaultPosition, wxDefaultSize,
					    MainFrame->GetCharsetsLen (),
					    MainFrame->GetCharsetsNames (),
					    wxLB_SINGLE);
    charsetCtrl->SetFont (wxFont
			  (8, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL,
			   wxFONTWEIGHT_NORMAL));
    int idSel = MainFrame->GetCharsetIndex (Charset);
    if (idSel != wxNOT_FOUND)
	charsetCtrl->SetSelection (idSel);
    charsetSizer->Add (charsetCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// second row: the ASK CHARSET
    wxBoxSizer *askSizer = new wxBoxSizer (wxHORIZONTAL);
    charsetSizer->Add (askSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxCheckBox *askCtrl = new wxCheckBox (this, ID_DFLT_ASK,
					  wxT ("Ask output charset every time"),
					  wxDefaultPosition, wxDefaultSize);
    askCtrl->SetValue (AskCharset);
    askSizer->Add (askCtrl, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & DefaultCharsetDialog::OnOk);
}

void
DefaultCharsetDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxListBox *charsetCtrl = (wxListBox *) FindWindow (ID_DFLT_CHARSET);
    int idSel = charsetCtrl->GetSelection ();
    if (idSel == wxNOT_FOUND)
      {
	  wxMessageBox (wxT
			("you must select some Default Charset Encoding from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxString *charsets = MainFrame->GetCharsets ();
    Charset = *(charsets + idSel);
    wxCheckBox *askCtrl = (wxCheckBox *) FindWindow (ID_DFLT_ASK);
    AskCharset = askCtrl->GetValue ();
    wxDialog::EndModal (wxID_OK);
}

bool
RecoverDialog::Create (MyFrame * parent, wxString & table, wxString & column)
{
//
// creating the dialog
//
    MainFrame = parent;
    Table = table;
    Column = column;
    Srid = -1;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Recover Geometry Column")) ==
	false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
RecoverDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// First row: TABLE name
    wxBoxSizer *tableSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (tableSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *tableLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Table name:"));
    tableSizer->Add (tableLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *tableCtrl = new wxTextCtrl (this, wxID_ANY, Table,
					    wxDefaultPosition, wxSize (350, 22),
					    wxTE_READONLY);
    tableCtrl->Enable (false);
    tableSizer->Add (tableCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: GEOMETRY COLUMN name
    wxBoxSizer *colSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (colSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *colLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Column name:"));
    colSizer->Add (colLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *colCtrl = new wxTextCtrl (this, wxID_ANY, Column,
					  wxDefaultPosition, wxSize (350, 22),
					  wxTE_READONLY);
    colCtrl->Enable (false);
    colSizer->Add (colCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// third row: SRID
    wxBoxSizer *sridSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (sridSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxStaticText *sridLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&SRID:"));
    sridSizer->Add (sridLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxSpinCtrl *sridCtrl = new wxSpinCtrl (this, ID_RCVR_SRID, wxEmptyString,
					   wxDefaultPosition, wxSize (80, 20),
					   wxSP_ARROW_KEYS,
					   -1, 40000, Srid);
    sridSizer->Add (sridCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// fourth row: GEOMETRY TYPE
    wxStaticBox *typeBox = new wxStaticBox (this, wxID_STATIC,
					    wxT ("Geometry Type"),
					    wxDefaultPosition, wxDefaultSize);
    wxBoxSizer *typeSizer = new wxStaticBoxSizer (typeBox, wxHORIZONTAL);
    sridSizer->Add (typeSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxString types[7];
    types[0] = wxT ("POINT");
    types[1] = wxT ("MULTIPOINT");
    types[2] = wxT ("LINESTRING");
    types[3] = wxT ("MULTILINESTRING");
    types[4] = wxT ("POLYGON");
    types[5] = wxT ("MULTIPOLYGON");
    types[6] = wxT ("GEOMETRYCOLLECTION");
    wxListBox *geomType = new wxListBox (this, ID_RCVR_TYPE,
					 wxDefaultPosition, wxDefaultSize, 7,
					 types, wxLB_SINGLE);
    typeSizer->Add (geomType, 0, wxALIGN_CENTER_VERTICAL | wxALL, 0);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & RecoverDialog::OnOk);
}

void
RecoverDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxSpinCtrl *sridCtrl = (wxSpinCtrl *) FindWindow (ID_RCVR_SRID);
    wxListBox *geomType = (wxListBox *) FindWindow (ID_RCVR_TYPE);
    Srid = sridCtrl->GetValue ();
    if (MainFrame->SridNotExists (Srid) == true)
      {
	  wxMessageBox (wxT ("invalid SRID value"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    Type = geomType->GetStringSelection ();
    if (Type.Len () < 1)
      {
	  wxMessageBox (wxT
			("you must select some Geometry Type from the list"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxDialog::EndModal (wxID_OK);
}

bool
SetSridDialog::Create (MyFrame * parent, wxString & table, wxString & column)
{
//
// creating the dialog
//
    MainFrame = parent;
    Table = table;
    Column = column;
    OldSrid = -1;
    Srid = -1;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("Changing SRID")) == false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
SetSridDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// First row: TABLE name
    wxBoxSizer *tableSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (tableSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *tableLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Table name:"));
    tableSizer->Add (tableLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *tableCtrl = new wxTextCtrl (this, wxID_ANY, Table,
					    wxDefaultPosition, wxSize (350, 22),
					    wxTE_READONLY);
    tableCtrl->Enable (false);
    tableSizer->Add (tableCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// second row: GEOMETRY COLUMN name
    wxBoxSizer *colSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (colSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *colLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Column name:"));
    colSizer->Add (colLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *colCtrl = new wxTextCtrl (this, wxID_ANY, Column,
					  wxDefaultPosition, wxSize (350, 22),
					  wxTE_READONLY);
    colCtrl->Enable (false);
    colSizer->Add (colCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// third row: SRID
    wxBoxSizer *mySizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (mySizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxBoxSizer *oldSridSizer = new wxBoxSizer (wxHORIZONTAL);
    mySizer->Add (oldSridSizer, 0, wxALIGN_LEFT | wxALL, 0);
    wxStaticText *oldSridLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&old SRID:"));
    oldSridSizer->Add (oldSridLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxSpinCtrl *oldSridCtrl = new wxSpinCtrl (this, ID_SRID_OLD, wxEmptyString,
					      wxDefaultPosition, wxSize (80,
									 20),
					      wxSP_ARROW_KEYS,
					      -1, 40000, OldSrid);
    oldSridSizer->Add (oldSridCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// fourth row: SRID
    wxBoxSizer *sridSizer = new wxBoxSizer (wxHORIZONTAL);
    mySizer->Add (sridSizer, 0, wxALIGN_RIGHT | wxLEFT, 20);
    wxStaticText *sridLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&new SRID:"));
    sridSizer->Add (sridLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxSpinCtrl *sridCtrl = new wxSpinCtrl (this, ID_SRID_SRID, wxEmptyString,
					   wxDefaultPosition, wxSize (80, 20),
					   wxSP_ARROW_KEYS,
					   -1, 40000, Srid);
    sridSizer->Add (sridCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & SetSridDialog::OnOk);
}

void
SetSridDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxSpinCtrl *oldSridCtrl = (wxSpinCtrl *) FindWindow (ID_SRID_OLD);
    wxSpinCtrl *sridCtrl = (wxSpinCtrl *) FindWindow (ID_SRID_SRID);
    OldSrid = oldSridCtrl->GetValue ();
    Srid = sridCtrl->GetValue ();
    wxDialog::EndModal (wxID_OK);
}

bool
SearchSridDialog::Create (MyFrame * parent)
{
//
// creating the dialog
//
    MainFrame = parent;
    if (wxDialog::Create (parent, wxID_ANY,
			  wxT ("Searching a SRID by its name")) == false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    Centre ();
    return true;
}

void
SearchSridDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
// First row: SEARCH string
    wxBoxSizer *searchSizer = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (searchSizer, 0, wxALIGN_RIGHT | wxALL, 0);
    wxStaticText *searchLabel =
	new wxStaticText (this, wxID_STATIC, wxT ("&Search:"));
    searchSizer->Add (searchLabel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxTextCtrl *searchCtrl = new wxTextCtrl (this, ID_SEARCH, wxT (""),
					     wxDefaultPosition, wxSize (150,
									22));
    searchSizer->Add (searchCtrl, 0, wxALIGN_RIGHT | wxALL, 5);
// OK - CANCEL buttons
    wxBoxSizer *okCancelBox = new wxBoxSizer (wxHORIZONTAL);
    boxSizer->Add (okCancelBox, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);
    wxButton *ok = new wxButton (this, wxID_OK, wxT ("&OK"));
    okCancelBox->Add (ok, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    wxButton *cancel = new wxButton (this, wxID_CANCEL, wxT ("&Cancel"));
    okCancelBox->Add (cancel, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
// appends event handler for OK button
    Connect (wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
	     (wxObjectEventFunction) & SearchSridDialog::OnOk);
}

void
SearchSridDialog::OnOk (wxCommandEvent & event)
{
//
// all done: 
//
    wxTextCtrl *searchCtrl = (wxTextCtrl *) FindWindow (ID_SEARCH);
    String = searchCtrl->GetValue ();
    if (String.Len () < 1)
      {
	  wxMessageBox (wxT ("You must specify a string to search for !!!"),
			wxT ("spatialite-gui"), wxOK | wxICON_WARNING, this);
	  return;
      }
    wxDialog::EndModal (wxID_OK);
}

bool
HelpDialog::Create (MyFrame * parent)
{
//
// creating the dialog
//
    MainFrame = parent;
    if (wxDialog::Create (parent, wxID_ANY, wxT ("SQLite + SpatiaLite help")) ==
	false)
	return false;
// populates individual controls
    CreateControls ();
// sets dialog sizer
    GetSizer ()->Fit (this);
    GetSizer ()->SetSizeHints (this);
// centers the dialog window
    MainFrame->OpenHelpPane ();
    Centre ();
// setting up an event handler [dialog closing]
    Connect (wxID_ANY, wxEVT_CLOSE_WINDOW,
	     (wxObjectEventFunction) & HelpDialog::OnClose);
    return true;
}

void
HelpDialog::OnClose (wxCloseEvent & event)
{
//
// this window has been closed
//
    MainFrame->CloseHelpPane ();
    Destroy ();
}

void
HelpDialog::CreateControls ()
{
//
// creating individual control and setting initial values
//
    wxBoxSizer *topSizer = new wxBoxSizer (wxVERTICAL);
    this->SetSizer (topSizer);
    wxBoxSizer *boxSizer = new wxBoxSizer (wxVERTICAL);
    topSizer->Add (boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    wxHtmlWindow *helpWin = new wxHtmlWindow (this, wxID_ANY,
					      wxDefaultPosition, wxSize (620,
									 420));
    wxString html;
    MainFrame->GetHelp (html);
    helpWin->SetPage (html);
    topSizer->Add (helpWin, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
}
